/*******************************************
Dead E. Ruxpin by Cameron Coward
cameroncoward.com

Frequecy chart:
1000Hz  (Band 0): open mouth
1351Hz  (Band 1): close mouth
1825Hz (Band 2): lift left arm
2466Hz (Band 3): lower left arm
3331Hz (Band 4): lift right arm
4500Hz (Band 5): lower right arm

*******************************************/

#include <arduinoFFT.h>
#include <Wire.h>
#include <Adafruit_PWMServoDriver.h>

#define SAMPLES 2048        // Number of samples for FFT
#define SAMPLING_FREQ 10000 // Sampling frequency in Hz
#define NUM_BANDS 6
#define NOISE 300           // Used as a crude noise filter, values below this are ignored

#define AUDIO_IN_PIN A0       // Analog input pin

// Sampling and FFT stuff
unsigned int sampling_period_us;
int bandValues[] = {0,0,0,0,0,0};
double vReal[SAMPLES];
double vImag[SAMPLES];
unsigned long newTime;
arduinoFFT FFT = arduinoFFT(vReal, vImag, SAMPLES, SAMPLING_FREQ);

// Servo controller stuff
Adafruit_PWMServoDriver pwm = Adafruit_PWMServoDriver(0x40);
#define MOUTHMIN  725 // This is the rounded 'minimum' microsecond length based on the minimum pulse of 150
#define MOUTHMAX  1200 // This is the rounded 'maximum' microsecond length based on the maximum pulse of 600
#define LEFTMIN  2000 // This is the rounded 'minimum' microsecond length based on the minimum pulse of 150
#define LEFTMAX  1500 // This is the rounded 'maximum' microsecond length based on the maximum pulse of 600
#define RIGHTMIN  1000 // This is the rounded 'minimum' microsecond length based on the minimum pulse of 150
#define RIGHTMAX  1500 // This is the rounded 'maximum' microsecond length based on the maximum pulse of 600
#define SERVO_FREQ 50 // Analog servos run at ~50 Hz updates


void setup() {
  Serial.begin(9600);
  sampling_period_us = round(1000000 * (1.0 / SAMPLING_FREQ));
  pwm.begin();
  pwm.setOscillatorFrequency(27000000);
  pwm.setPWMFreq(SERVO_FREQ);  // Analog servos run at ~50 Hz updates
  pwm.writeMicroseconds(0, MOUTHMIN);
  pwm.writeMicroseconds(1, LEFTMIN);
  pwm.writeMicroseconds(2, RIGHTMIN);
}

void loop() {
  for (int i = 0; i<NUM_BANDS; i++){
    bandValues[i] = 0;
  }

  // Sample the audio pin
  for (int i = 0; i < SAMPLES; i++) {
    newTime = micros();
    vReal[i] = analogRead(AUDIO_IN_PIN); // A conversion takes about 9.7uS on an ESP32
    vImag[i] = 0;
    while ((micros() - newTime) < sampling_period_us) { /* chill */ }
  }

  // Compute FFT
  FFT.DCRemoval();
  FFT.Windowing(FFT_WIN_TYP_HAMMING, FFT_FORWARD);
  FFT.Compute(FFT_FORWARD);
  FFT.ComplexToMagnitude();

  // Analyse FFT results
  for (int i = 2; i < (SAMPLES/2); i++){       // Don't use sample 0 and only first SAMPLES/2 are usable. Each array element represents a frequency bin and its value the amplitude.
    if (vReal[i] > NOISE) {                    // Add a crude noise filter

    // Store center sample of each the four bands
      if (i>150   && i<=241  ) bandValues[0]  += (int)vReal[i];
      if (i>241   && i<=325  ) bandValues[1]  += (int)vReal[i];
      if (i>325   && i<=439  ) bandValues[2]  += (int)vReal[i];
      if (i>439   && i<=594  ) bandValues[3]  += (int)vReal[i];
      if (i>594   && i<=802  ) bandValues[4]  += (int)vReal[i];
      if (i>802   && i<=1070 ) bandValues[5]  += (int)vReal[i];

    }
  }

  Serial.print("Band 0 (500Hz):");
  Serial.print(bandValues[0]);
  Serial.print(",");
  Serial.print("Band 1 (758Hz):");
  Serial.print(bandValues[1]);
  Serial.print(",");
  Serial.print("Band 2 (1149Hz):");
  Serial.print(bandValues[2]);
  Serial.print(",");
  Serial.print("Band 3 (1741Hz):");
  Serial.print(bandValues[3]);
  Serial.print(",");
  Serial.print("Band 4 (2639Hz):");
  Serial.print(bandValues[4]);
  Serial.print(",");
  Serial.print("Band 5 (4000Hz):");
  Serial.println(bandValues[5]);

  if (bandValues[0] > 16000) pwm.writeMicroseconds(0, MOUTHMAX);
  if (bandValues[1] > 16000) pwm.writeMicroseconds(0, MOUTHMIN);
  if (bandValues[2] > 16000) pwm.writeMicroseconds(1, LEFTMAX);
  if (bandValues[3] > 16000) pwm.writeMicroseconds(1, LEFTMIN);
  if (bandValues[4] > 5000) pwm.writeMicroseconds(2, RIGHTMAX);
  if (bandValues[5] > 5000) pwm.writeMicroseconds(2, RIGHTMIN);

}